" Maintainer:	Peter Valach <pvalach@gmx.net>
" Last Changed: 2002 Nov 5
" URL:		http://www.pv2c.sk/free/sk_vimkeymap/

let b:keymap_name = "sk"

loadkeymap
" Z a Y - ak chcete slovensku qwerty, zakomentujte tieto styri riadky
y z
z y
Y Z
Z Y

" Horny riadok
` ;
1 +
! 1
2 ľ
@ 2
3 š
# 3
4 č
$ 4
5 ť
% 5
6 ž
^ 6
7 ý
& 7
8 á
* 8
9 í
( 9
0 é
) 0
- =
_ %

" Klavesy okolo Enteru :)
[ ú
{ /
] ä
} (
\\ ň
<Bar> )
; ô
: "
' §
\" !
", , ;-)
< ?
". . ;-)
> :
/ -
? _

" Dlzne
=a á
=A Á
=e é
=E É
=i í
=I Í
=l ĺ
=L Ĺ
=o ó
=O Ó
=r ŕ
=R Ŕ
=u ú
=U Ú
=z ý
=Z Ý
" pre pripad omylu :) alebo slovenskej QWERTY
=y ý
=Y Ý

" Makcene
+a ä
+A Ä
+c č
+C Č
+d ď
+D Ď
+l ľ
+L Ľ
+n ň
+N Ň
+o ô
+O Ô
+s š
+S Š
+t ť
+T Ť
+y ž
+Y Ž
" pre pripad omylu :) alebo slovenskej QWERTY
+z ž
+Z Ž

